'use strict';

module.exports = function(app, options) {

	var express = require('express');
	var router = express.Router();
	var template = require('es6-template-strings');
	var webRequest = require('request');
	var _ = require('lodash');
	var GetUserInfoURL = "https://sim.dxy.cn/japi/open/qiye/contact/getbyticket";
	var Ticket;
	/**
	 * 根据ticket获取员工信息
	 */
	function GetUserInfo(ticket, cb) {
		var dict = BuildParameters();
		var str = ToSignString(dict);
		var sign = SHA1HashStringForUTF8String(str);
		dict.push({
			key: 'sign',
			value: sign
		});
		_.remove(dict, function(obj) {
			return obj.key == 'appSignKey';
		});
		var postData = ToSignString(dict);
		console.log(GetUserInfoURL + '?' + postData);
		webRequest.get(GetUserInfoURL + '?' + postData, function(error, response, body) {
			if (error) {
				return cb(error, null);
			}
			var data = JSON.parse(body);
			if (data.success) {
				if (data.item != undefined) {
					var User = data.item;
					return cb(null, User);
				} else {
					return cb({
						message: '身份信息为空！'
					}, null);
				}
			} else {
				return cb(data, null);

			}
		});

	}
	/**
	 * 授权回调
	 */
	router.get('/oauthcallback', function(req, res) {
		var ticket = req.query.ticket;
		if (!ticket) {
			return res.send('参数不完整');
		}
		Ticket = ticket;
		GetEmployeeInfo(ticket, req, res);
	});
	/**
	 * 生成参数
	 */
	function BuildParameters() {
		var dict = [{
			'key': 'appSignKey',
			'value': 'Ed6WRgR3xpznCVnzR60MluFGzFt6M0jMq15uOGypBhBGuUsdePnTjMDGQjpi042JweH5uBa2iBmGysxyMIoG73TxJbrrfUXSWVJMAd3sRV36eQfCQkkwQrN96nBk4fWd'
		}, {
			'key': 'nonce',
			'value': Random(16)
		}, {
			'key': 'ticket',
			'value': Ticket
		}, {
			'key': 'timestamp',
			'value': GetTimestamp()
		}, {
			'key': 'appId',
			'value': '1047972893'
		}];

		return dict;
	}
	/**
	 *生成时间戳 
	 */
	function GetTimestamp() {
		var timestamp = Date.parse(new Date());
		return timestamp.toString().replace('000', '');
	}
	/**生成随机字符串
	 * 
	 * @param {Object} length
	 */
	function Random(length) {
		var chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
		var stringChars = new Array(length);
		for (var i = 0; i < stringChars.length; i++) {
			var r = parseInt(Math.random() * chars.length);
			stringChars[i] = chars[r];
		}
		return stringChars.join('');
	}
	/**
	 * #根据生成的参数连接成字符串
	 */
	function ToSignString(dict) {
		return _.chain(dict).sortBy('key')
			.map(function(obj) {
				return obj.key + '=' + obj.value;
			})
			.join("&")
			.value();
	}
	/**
	 * 转成sha1
	 */
	function SHA1HashStringForUTF8String(str) {
		var sha1 = require('sha1');
		return sha1(str).toString();
	}
	/**
	 * 授权
	 */
	router.get('/oauth', function(req, res) {
		var redirect = "http://voice.66new.cn/callback?ticket=";
		return res.redirect("https://sim.dxy.cn/japi/qiye/oauth/ticket/148/" + toBase64(redirect));
	});

	/**
	 * 转换成BASE64
	 */
	function toBase64(arg1) {
		var str = new Buffer(arg1);
		var Key = str.toString('base64').replace(/\+/g, "-").replace(/\//g, "_");
		return _.trimEnd(Key, '=').toString();
	}
	return router;
}